require "defines"
require "util"

local loaded
local prefix = "   Items : "

-- We only care about roboports with a construction radius.
function Robocheck(obj)
	if obj.type == "roboport" and obj.logistic_cell.construction_radius > 1 then return true end
end
		
-- On Tick event
-- Every second to deconstruct
-- A few times a second for the gui check
function ticker()
	-- if global.RoboInfo ~= nil then
		if global.ticks == 0 or global.ticks == nil then
			global.ticks = 59
			processRobos()
		else
			global.ticks = global.ticks - 1
		end
		if (global.ticks - 10) % 20 == 0 then
			for i, player in ipairs(game.players) do
				showOrHideUI(player)
			end
		end
	-- end
end

function showOrHideUI(player)
	local obj = player.opened
	local show = obj ~= nil and Robocheck(obj)
	
	if show then
		-- Show checkbox if not already shown
		if not global.GuiRoboOpen then
			local collecting = obj.backer_name:find(prefix) ~= nil
			global.GuiRoboOpen = obj
			global.GuiRoboCollectState = collecting
			player.gui.top.add{type = "checkbox", name = "robo-collect", caption = "Mark all items on the ground for deconstruction?", state = collecting}
		end 
	else
		
		if player.gui.top["robo-collect"] then
			local newState = player.gui.top["robo-collect"].state
			if global.GuiRoboCollectState ~= newState then 
				if newState then AddToRoboList(global.GuiRoboOpen) else RemoveFromRoboList(global.GuiRoboOpen) end
			end
			player.gui.top["robo-collect"].destroy()
			global.GuiRoboOpen = nil
			global.GuiRoboCollectState = nil
		end
		
		
	end
end

-- Add to the Roboport info list. Remember & adjust the backer name
function AddToRoboList(Robo)
	global.RoboInfo = global.RoboInfo or {}
	local t = {}
	t.Robo = Robo
	t.OldBackerName = Robo.backer_name
	Robo.backer_name = prefix.."0"
	table.insert(global.RoboInfo, t)
end

-- Roboport deselected  - Remove from list/replace backer neame
function RemoveFromRoboList(Robo)
	-- Find it
	for i, R in ipairs(global.RoboInfo) do
		if Robo == R.Robo then
			Robo.backer_name = R.OldBackerName
			table.remove(global.RoboInfo, i)
			return
		end
	end
	
end

-- Deconstruct all the things. (on the floor)
function processRobos()
	if not global.RoboInfo then return end
	-- Process (backwards)
	for i = #global.RoboInfo,1,-1 do
		local Robo = global.RoboInfo[i].Robo
		if Robo.valid then
			local count = 0
			-- 'ports in the list will reset to zero if name invalid.
			if (#Robo.backer_name > #prefix) then
				count = tonumber(Robo.backer_name:sub(#prefix + 1))
				count = count or 0
			end
			
			-- Collect all items on the ground
			local r = Robo.logistic_cell.construction_radius
			local ents = Robo.surface.find_entities_filtered{
				area = {{Robo.position.x-r, Robo.position.y-r}, {Robo.position.x+r, Robo.position.y+r}},
				name = "item-on-ground"}
				
			-- Only deconstruct items not already marked. (otherwise you end up with multiple robots going after the same item)
			for i, item in ipairs(ents) do
				if not item.to_be_deconstructed(Robo.force) then 
					item.order_deconstruction(Robo.force)
					count = count + 1
				end
			end
			
			-- Backer name - now the counter
			if #ents > 0 then 
				Robo.backer_name = prefix..tostring(count)
			end
		else
			-- Deleted
			table.remove(global.RoboInfo, i)
		end
	end
end


script.on_init(function()
	loaded = true
	script.on_event(defines.events.on_tick, ticker)
end)

script.on_load(function()
	if not loaded then
		loaded = true
		script.on_event(defines.events.on_tick, ticker)
	end
end)

		